//
// Created by hoelshare on 21.05.17.
//
#include "LaserMenuItem.hpp"
#include <FS.h>
#include <TFT_ILI9163C.h>
#include "BadgeUI.h"

uint32_t read32(File &f);

uint16_t read16(File &f);

void LaserMenuItem::bmpDraw(uint8_t x, uint16_t y, TFT_ILI9163C *tft) {
    if (this->imagePath) {
        File bmpFile;
        uint16_t bmpWidth, bmpHeight;   // W+H in pixels
        uint8_t bmpDepth;              // Bit depth (currently must be 24)
        uint32_t bmpImageoffset;        // Start of image data in file
        uint32_t rowSize;               // Not always = bmpWidth; may have padding
        uint8_t sdbufferLen = BUFFPIXEL * 3;
        uint8_t sdbuffer[sdbufferLen]; // pixel buffer (R+G+B per pixel)
        uint8_t buffidx = sdbufferLen; // Current position in sdbuffer
        boolean goodBmp = false;       // Set to true on valid header parse
        boolean flip = true;        // BMP is stored bottom-to-top
        uint16_t w, h, row, col;
        uint8_t r, g, b;
        uint32_t pos = 0;

        if ((x >= tft->width()) || (y >= tft->height())) return;

        // Open requested file on SD card
        if ((bmpFile = SPIFFS.open(this->imagePath, "r")) == NULL) {
            // TODO Download File
            return;
        }

        // Parse BMP header
        if (read16(bmpFile) == 0x4D42) { // BMP signature
            read32(bmpFile);
            (void) read32(bmpFile); // Read & ignore creator bytes
            bmpImageoffset = read32(bmpFile); // Start of image data
            // Read DIB header
            read32(bmpFile);
            bmpWidth = read32(bmpFile);
            bmpHeight = read32(bmpFile);
            if (read16(bmpFile) == 1) { // # planes -- must be '1'
                bmpDepth = read16(bmpFile); // bits per pixel
                if ((bmpDepth == 24) && (read32(bmpFile) == 0)) { // 0 = uncompressed
                    goodBmp = true; // Supported BMP format -- proceed!
                    rowSize = (bmpWidth * 3 + 3) & ~3;// BMP rows are padded (if needed) to 4-byte boundary
                    if (bmpHeight < 0) {
                        bmpHeight = -bmpHeight;
                        flip = false;
                    }
                    // Crop area to be loaded
                    w = bmpWidth;
                    h = bmpHeight;
                    if ((x + w - 1) >= tft->width()) w = tft->width() - x;
                    if ((y + h - 1) >= tft->height()) h = tft->height() - y;
                    //tft->startPushData(x, y, x+w-1, y+h-1);
                    for (row = 0; row < h; row++) { // For each scanline...
                        uint8_t row_data[w * 2];
                        if (flip) { // Bitmap is stored bottom-to-top order (normal BMP)
                            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
                        } else {     // Bitmap is stored top-to-bottom
                            pos = bmpImageoffset + row * rowSize;
                        }
                        if (bmpFile.position() != pos) { // Need seek?
                            bmpFile.seek(pos, SeekSet);
                            buffidx = sdbufferLen; // Force buffer reload
                        }
                        for (col = 0; col < w; col++) { // For each pixel...
                            // Time to read more pixel data?
                            if (buffidx >= sdbufferLen) { // Indeed
                                bmpFile.read(sdbuffer, sdbufferLen);
                                buffidx = 0; // Set index to beginning
                            }
                            // Convert pixel from BMP to TFT format, push to display
                            b = sdbuffer[buffidx++];
                            g = sdbuffer[buffidx++];
                            r = sdbuffer[buffidx++];
                            uint16_t val = tft->Color565(r, g, b);
                            row_data[2 * col + 1] = (uint8_t) val;
                            row_data[2 * col] = *(((uint8_t * ) & val) + 1);
                        } // end pixel
                        tft->writeRow(y + row, x, w, row_data);
                    } // end scanline
                    //tft->endPushData();
                } // end goodBmp
            }
        }

        bmpFile.close();
        if (!goodBmp) {
            tft->setCursor(20, 20);
            tft->print("file unrecognized!");
        }
    }
}
